import com.swath.*;
import com.swath.cmd.*;

/*
	Title: My Planet Buster v1.0b
	Author: Myoptik (1st JAVA Script), myoptik@yahoo.com
	Built: FEB 06
	
	Synop:	Not the fastest, but very functional Planet Buster. Some selling points:
		Pre-Calculates maximum experience that can be gained based on Credits on hand.
		Will not buy more Atomic Devices than required to reach goal.

	Ideas:	evasion tactics.

	Fixes:	Landing with a ship equipped with a Planet-scanner didn't work. Implemented the
		ScanPlanets method. 

	Thanks to Mind Dagger for the plethora of code to study.
*/

public class MyPBuster extends UserDefinedScript {
	protected static final String DEF_PLANET_NOM = "Yippie Kia Yea!";
	protected static final int ATOMIC_COST = 15000;
	protected static final int TORPEDO_COST = 20000;
	protected static final int MAX_EXP = 4194304;		// Maximum Possible Exp
	private int ibought = 0;
	private int i;
	private int iStartEXP;				// Exp at start
	private int iStartCRD;				// Credits on hand
	private boolean bHasPScan;				// Indicates a Planet Scanner
	private boolean bFast;
	private int iIterations;				// Number of bangs possible (based upon iStartCRD)
	private Parameter iTargetExp;			// User desired Exp level
	private Parameter sNom;				// Name ot give buster planets
	private Planet[] planets;
	
	public String getName() {
		return "My Planet Buster 1.0b";
	}

	public String getDescription() {
		return "My Planet Buster Version 1.00a\n\nAsks for target experience and Busts accordingly.\n\n"+
			"Pre-Calculates maximum experience that can be gained based on Credits on hand.\n\n"+
			"Will not buy more Atomic Devices than required to reach goal.\n\n"+
			"Created by Myoptik - FEB 06";	
	}

	public boolean initScript() throws Exception {
		/*		Do basic checks before proceeding...			*/
		if (!atPrompt(Swath.COMMAND_PROMPT)) {				// Make sure we're at CMD Prompt;
			throw new ScriptException("Not at correct prompt");	}

		if(Swath.main.currSector() != Swath.main.stardock()) {			// ...located at SD Sector;
			throw new ScriptException("Not at Stardock.");		}

		if(Swath.ship.shipCategory().maxGenesisTorpedos() == 0) {			// ...ship can use Torpedoes;
			if(TorpedosOnHand() == 0) {					//
				MessageBox.exec("It appears your Ship cannot house Torpedoes.\n\nIf this is not the case, view "+
						"your Ship in the catalog to update Swath's DBase, or, buy at least"+
						"on Torpedoe from Star Dock and try again.", "Buster - Ship Error.", MessageBox.ICON_EXCLAMATION, MessageBox.TYPE_OK );
				throw new ScriptException("Cannot house Torpedos in current ship.");	
			}
		}
		
		iStartEXP = Swath.you.experience();
		iStartCRD = Swath.you.credits();
		bHasPScan = Swath.ship.hasPlanetScanner();

		if(iStartCRD <  (ATOMIC_COST+TORPEDO_COST)) {			// ...enough money on hand.
			throw new ScriptException("Not Enough Money. Get busy and make some.");	}

		iIterations = (iStartCRD / (ATOMIC_COST+TORPEDO_COST));

		i = (iStartEXP + (iIterations * 75));		// Get max experience to be gained

		if (i > MAX_EXP) {				// Not likely to ever be a problem, but you never know
			iIterations = ((i - MAX_EXP) / 75)-1;	// recalc iteration value
			i = (iIterations * 75);			// new Expereince value
		} 					//**Note to self: implement Atomics on hand into calculation

		iTargetExp = new Parameter("Target Experience");
		iTargetExp.setType(Parameter.INTEGER);
		iTargetExp.setHelpText("Reaching the intended Experience depends largly on amount of Credits on hand.");
		iTargetExp.setInteger(i);					//Should 
		iTargetExp.setIntegerRange	(iStartEXP, MAX_EXP);		//Max Exp in game

		sNom = new Parameter("Planets Name");
		sNom.setType(Parameter.STRING);
		sNom.setString(DEF_PLANET_NOM);
		sNom.setMaxStringLength(40);		// Max set in game
		sNom.setHelpText("Customize the name of Busted Planets.");

		registerParam(iTargetExp);	
		registerParam(sNom);

		return true;
	}	
	
	public boolean runScript() throws Exception {
		int i=0;

		DisplayCurrentInfo.exec(true);					// true - display quick info

		EnterComputer.exec();
                                ChangeSettings.exec(ChangeSettings.ALL_KEYS);
		LeaveComputer.exec();

		if (iTargetExp.getInteger() <= iStartEXP) {
			throw new ScriptException("Nothing to do : Target Exp not high enough.");	
		}

		iIterations = ((iTargetExp.getInteger() - iStartEXP) / 75);

		while( i < iIterations){
			if((AtomicsOnHand()==0) || (TorpedosOnHand()==0)) Restock();
			Genesize();
			GoBoom();
			i++;
		}

		EnterComputer.exec();
                                ChangeSettings.exec(ChangeSettings.SPACE);
		LeaveComputer.exec();

		return true;
	}

	public void endScript(boolean finished) throws Exception {
		// Nadda to do
	}

	// Not really necessary, but allows for future development.
	public int AtomicsOnHand() throws Exception {
		return Swath.ship.atomicDevices();			}

	public int TorpedosOnHand() throws Exception {
		return Swath.ship.genesisTorpedos();		}


	public boolean Restock() throws Exception {				// Assumes: at Stardock Sector && Required cash is on hand
		int ibuy;

		if ((iIterations-ibought) >=5) {					// Keep track so as not to buy too many items
			ibuy=5;
		}
		else {
			ibuy=(iIterations-ibought);
		}

		SendMacro.exec("p s h a "+ibuy+"^m t "+ibuy+"^m q q/", 1, false);	// Run once, Express=false
		ibought += ibuy;						// remember purchased amount
		return true;				}	


	public boolean Genesize() throws Exception {
		if (sNom.equals("")) {
			sNom.setString(DEF_PLANET_NOM);			// I wonder: does busting Corporate planets 
		}							// increase Corporate Experience as well (?)

		if (Swath.you.corporation() != null) {
			SendMacro.exec("U Y "+sNom.getString()+"^mC", 1, false);
		}
		else {
			SendMacro.exec("U Y "+sNom.getString()+"^mP", 1, false);
		}
		return true;				}

	public boolean GoBoom() throws Exception {
		if (bHasPScan) {
			planets = ScanPlanets.exec();
			
			while(Swath.main.prompt() != Swath.COMMAND_PROMPT) {
				Thread.sleep(25);
			}
			
			if(planets.length == 0) throw new ScriptException("Nothing to Blow Up : No planet(s) found.");	

			for(int i=0; i < planets.length;i++)
			{
				if((planets[i].owner().isYou()) || (planets[i].owner().isYourCorporation()) ) {
					Planet currentPlanet = (Planet)(planets[i]);
					SendMacro.exec("L "+currentPlanet.id()+"^m Z D Y ", 1, false);
					//SendSSRadioMessage.exec("PLanet.id " + currentPlanet.id());
				}
			}
		}
		else {
			SendMacro.exec("L Z D Y", 1, false);	
		}

		return true;				}
}
		// This is the end
		// My only friend
		// The end