import com.swath.*;

/**
 * This is my first SWATH daemon.
 * It demonstrates how to use the timer and how to receive events.
 * Every 15 seconds it will display how many fighters the ship has.
 * 
 * @author Stein
 * @since SWATH 1.7
 */
public class ExampleDaemon1 extends UserDefinedDaemon {
	public String getName() {
		// Return the name of the daemon
		return "Example Daemon #1";
	}

	public boolean initDaemon() throws Exception {
		// Initialisation of the daemon is done in this method.
		// If something goes wrong, throw a DaemonException or return false.
		Swath.printTrace("initDaemon()");

		// Enable processing of events
		enableEvents();

		// Enable processing of incoming text
		//enableIncomingText();

		// Set the timer to trigger immediately
		setTimer(0);

		return true;
	}

	public void endDaemon() throws Exception {
		// Clean up after the daemon if needed
	}

	public void onEvent(EventIfc event) throws Exception {
		// Here you can receive and process incoming events
		Swath.printTrace("onEvent('" + event.getClassName() + "')");
	}

	public void onTimer(java.util.Date time) throws Exception {
		// This method will be called if the timer is triggered
		Swath.printTrace("onTimer('" + time + "')");

		// Show how many fighters we have
		Swath.userAlert("Fighters: " + Swath.ship.fighters(), Swath.ALERT_INFORMATION);

		// Set the timer to trigger again in 15 seconds from now
		setTimer(15 * 1000);
	}

	public void onText(String buffer, String text) throws Exception {
		// This method will be called when new text has arrived
		Swath.printTrace("onText('" + text + "')");
	}
}
